@@ -1,26 +1,57 @@
-var fs   = require('fs'),
-    path = require('path'),
-    cnst = require('constants');
+let fs   = require('fs');
+let path = require('path');
+let cnst = require('constants');
 
-var rimraf     = require('rimraf'),
-    osTmpdir   = require('os-tmpdir'),
-    rimrafSync = rimraf.sync;
+let os         = require('os');
+let rimraf     = require('rimraf');
+let mkdirp     = require('mkdirp');
+let osTmpdir   = require('os').tmpdir();
 
-/* HELPERS */
+const rimrafSync = rimraf.sync;
 
-var RDWR_EXCL = cnst.O_CREAT | cnst.O_TRUNC | cnst.O_RDWR | cnst.O_EXCL;
+//== helpers
+//
+let dir = path.resolve(os.tmpdir());
+
+let RDWR_EXCL = cnst.O_CREAT | cnst.O_TRUNC | cnst.O_RDWR | cnst.O_EXCL;
+
+let promisify = function(callback) {
+  if (typeof callback === 'function') {
+    return [undefined, callback];
+  }
+
+  var promiseCallback;
+  var promise = new Promise(function(resolve, reject) {
+    promiseCallback = function() {
+      var args = Array.from(arguments);
+      var err = args.shift();
+
+      process.nextTick(function() {
+        if (err) {
+          reject(err);
+        } else if (args.length === 1) {
+          resolve(args[0]);
+        } else {
+          resolve(args);
+        }
+      });
+    };
+  });
+
+  return [promise, promiseCallback];
+};
 
 var generateName = function(rawAffixes, defaultPrefix) {
   var affixes = parseAffixes(rawAffixes, defaultPrefix);
   var now = new Date();
   var name = [affixes.prefix,
-              now.getYear(), now.getMonth(), now.getDate(),
+              now.getFullYear(), now.getMonth(), now.getDate(),
               '-',
               process.pid,
               '-',
               (Math.random() * 0x100000000 + 1).toString(36),
               affixes.suffix].join('');
-  return path.join(affixes.dir || exports.dir, name);
+  return path.join(affixes.dir || dir, name);
 };
 
 var parseAffixes = function(rawAffixes, defaultPrefix) {
@@ -73,7 +104,14 @@ function deleteDirOnExit(dirPath) {
 function attachExitListener() {
   if (!tracking) return false;
   if (!exitListenerAttached) {
-    process.addListener('exit', cleanupSync);
+    process.addListener('exit', function() {
+        try {
+            cleanupSync();
+        } catch(err) {
+            console.warn("Fail to clean temporary files on exit : ", err);
+            throw err;
+        }
+    });
     exitListenerAttached = true;
   }
 }
@@ -85,26 +123,26 @@ function cleanupFilesSync() {
   var count = 0;
   var toDelete;
   while ((toDelete = filesToDelete.shift()) !== undefined) {
-    rimrafSync(toDelete);
+    rimrafSync(toDelete, { maxBusyTries: 6 });
     count++;
   }
   return count;
 }
 
 function cleanupFiles(callback) {
+  var p = promisify(callback);
+  var promise = p[0];
+  callback = p[1];
+
   if (!tracking) {
-    if (callback) {
-      callback(new Error("not tracking"));
-    }
-    return;
+    callback(new Error("not tracking"));
+    return promise;
   }
   var count = 0;
   var left = filesToDelete.length;
   if (!left) {
-    if (callback) {
-      callback(null, count);
-    }
-    return;
+    callback(null, count);
+    return promise;
   }
   var toDelete;
   var rimrafCallback = function(err) {
@@ -115,22 +153,21 @@ function cleanupFiles(callback) {
     if (err) {
       // This shouldn't happen; pass error to callback and abort
       // processing
-      if (callback) {
-        callback(err);
-      }
+      callback(err);
       left = 0;
       return;
     } else {
       count++;
     }
     left--;
-    if (!left && callback) {
+    if (!left) {
       callback(null, count);
     }
   };
   while ((toDelete = filesToDelete.shift()) !== undefined) {
-    rimraf(toDelete, rimrafCallback);
+    rimraf(toDelete, { maxBusyTries: 6 }, rimrafCallback);
   }
+  return promise;
 }
 
 function cleanupDirsSync() {
@@ -140,26 +177,26 @@ function cleanupDirsSync() {
   var count = 0;
   var toDelete;
   while ((toDelete = dirsToDelete.shift()) !== undefined) {
-    rimrafSync(toDelete);
+    rimrafSync(toDelete, { maxBusyTries: 6 });
     count++;
   }
   return count;
 }
 
 function cleanupDirs(callback) {
+  var p = promisify(callback);
+  var promise = p[0];
+  callback = p[1];
+
   if (!tracking) {
-    if (callback) {
-      callback(new Error("not tracking"));
-    }
-    return;
+    callback(new Error("not tracking"));
+    return promise;
   }
   var count = 0;
   var left = dirsToDelete.length;
   if (!left) {
-    if (callback) {
-      callback(null, count);
-    }
-    return;
+    callback(null, count);
+    return promise;
   }
   var toDelete;
   var rimrafCallback = function (err) {
@@ -170,22 +207,21 @@ function cleanupDirs(callback) {
     if (err) {
       // rimraf handles most "normal" errors; pass the error to the
       // callback and abort processing
-      if (callback) {
-        callback(err, count);
-      }
+      callback(err, count);
       left = 0;
       return;
     } else {
-      count;
+      count++;
     }
     left--;
-    if (!left && callback) {
+    if (!left) {
       callback(null, count);
     }
   };
   while ((toDelete = dirsToDelete.shift()) !== undefined) {
-    rimraf(toDelete, rimrafCallback);
+    rimraf(toDelete, { maxBusyTries: 6 }, rimrafCallback);
   }
+  return promise;
 }
 
 function cleanupSync() {
@@ -198,86 +234,93 @@ function cleanupSync() {
 }
 
 function cleanup(callback) {
+  var p = promisify(callback);
+  var promise = p[0];
+  callback = p[1];
+
   if (!tracking) {
-    if (callback) {
-      callback(new Error("not tracking"));
-    }
-    return;
+    callback(new Error("not tracking"));
+    return promise;
   }
   cleanupFiles(function(fileErr, fileCount) {
     if (fileErr) {
-      if (callback) {
-        callback(fileErr, {files: fileCount})
-      }
+      callback(fileErr, {files: fileCount});
     } else {
       cleanupDirs(function(dirErr, dirCount) {
-        if (callback) {
-          callback(dirErr, {files: fileCount, dirs: dirCount});
-        }
+        callback(dirErr, {files: fileCount, dirs: dirCount});
       });
     }
   });
+  return promise;
 }
 
-/* DIRECTORIES */
+//== directories
+//
+const mkdir = (affixes, callback) => {
+  const p = promisify(callback);
+  const promise = p[0];
+  callback = p[1];
 
-function mkdir(affixes, callback) {
-  var dirPath = generateName(affixes, 'd-');
-  fs.mkdir(dirPath, 0700, function(err) {
+  let dirPath = generateName(affixes, 'd-');
+  mkdirp(dirPath, 0o700, (err) => {
     if (!err) {
       deleteDirOnExit(dirPath);
     }
-    if (callback) {
-      callback(err, dirPath);
-    }
+    callback(err, dirPath);
   });
+  return promise;
 }
 
-function mkdirSync(affixes) {
-  var dirPath = generateName(affixes, 'd-');
-  fs.mkdirSync(dirPath, 0700);
+const mkdirSync = (affixes) => {
+  let dirPath = generateName(affixes, 'd-');
+  mkdirp.sync(dirPath, 0o700);
   deleteDirOnExit(dirPath);
   return dirPath;
 }
 
-/* FILES */
+//== files
+//
+const open = (affixes, callback) => {
+  const p = promisify(callback);
+  const promise = p[0];
+  callback = p[1];
 
-function open(affixes, callback) {
-  var filePath = generateName(affixes, 'f-');
-  fs.open(filePath, RDWR_EXCL, 0600, function(err, fd) {
+  const path = generateName(affixes, 'f-');
+  fs.open(path, RDWR_EXCL, 0o600, (err, fd) => {
     if (!err) {
-      deleteFileOnExit(filePath);
-    }
-    if (callback) {
-      callback(err, {path: filePath, fd: fd});
+      deleteFileOnExit(path);
     }
+    callback(err, { path, fd });
   });
+  return promise;
 }
 
-function openSync(affixes) {
-  var filePath = generateName(affixes, 'f-');
-  var fd = fs.openSync(filePath, RDWR_EXCL, 0600);
-  deleteFileOnExit(filePath);
-  return {path: filePath, fd: fd};
+const openSync = (affixes) => {
+  const path = generateName(affixes, 'f-');
+  let fd = fs.openSync(path, RDWR_EXCL, 0o600);
+  deleteFileOnExit(path);
+  return { path, fd };
 }
 
-function createWriteStream(affixes) {
-  var filePath = generateName(affixes, 's-');
-  var stream = fs.createWriteStream(filePath, {flags: RDWR_EXCL, mode: 0600});
-  deleteFileOnExit(filePath);
+const createWriteStream =  (affixes) => {
+  const path = generateName(affixes, 's-');
+  let stream = fs.createWriteStream(path, { flags: RDWR_EXCL, mode: 0o600 });
+  deleteFileOnExit(path);
   return stream;
 }
 
-/* EXPORTS */
-// Settings
-exports.dir               = path.resolve(osTmpdir());
-exports.track             = track;
-// Functions
-exports.mkdir             = mkdir;
-exports.mkdirSync         = mkdirSync;
-exports.open              = open;
-exports.openSync          = openSync;
-exports.path              = generateName;
-exports.cleanup           = cleanup;
-exports.cleanupSync       = cleanupSync;
+//== settings
+//
+exports.dir = dir;
+exports.track = track;
+
+//== functions
+//
+exports.mkdir = mkdir;
+exports.mkdirSync = mkdirSync;
+exports.open = open;
+exports.openSync = openSync;
+exports.path = generateName;
+exports.cleanup = cleanup;
+exports.cleanupSync = cleanupSync;
 exports.createWriteStream = createWriteStream;
